# flake8: noqa
from unittest import TestCase

import mapactionpy_controller.data_source
import mapactionpy_controller.map_doc
import mapactionpy_controller.map_report
import mapactionpy_controller.map_result
import mapactionpy_controller.plugin_base
import mapactionpy_controller.plugin_controller


class TestAllStuffTransfered(TestCase):

    def setUp(self):
        pass

    def test_all_stuff_transfered(self):
        self.assertTrue(True)
