# from mapactionpy_controller.steps import Step

# # Load event
# Step(
#     myfunc,
#     'Checking that the Humanitarian Event description file opens correctly',
#     'The Humanitarian Event description file opened correctly',
#     'Failed to open the Humanitarian Event description file correctly',
# ),

# # Load CMF Desc
# Step(
#     myfunc,
#     'Checking that the Crash Move Folder description file opens correctly',
#     'The Crash Move Folder description file opened correctly',
#     'Failed to open the Crash Move Folder description file correctly',
# ),

# # Load Recipe(s)
# Step(
#     myfunc,
#     'Checking that the Map Recipe file opens correctly',
#     'The Map Recipe file opened correctly',
#     'Failed to open the Map Recipe file correctly',
# ),

# # Combine recipe with layer_props
# Step(
#     myfunc,
#     'Combining Map Recipe with Layer Properties',
#     'Combined Map Recipe with Layer Properties',
#     'Failed to combined Map Recipe with Layer Properties',
# ),

# # Combine Recipe and event
# Step(
#     myfunc,
#     'Combining Map Recipe with Humanitarian Event',
#     'Combined Map Recipe with Humanitarian Event',
#     'Failed to combined Map Recipe with Humanitarian Event',
# ),

# # Check naming convention for all data in 2_Active_Data

# # Check that each file in the right sub folder within 2_Active_Data


# #  - update regexs
# Step(
#     myfunc,
#     'Updating data search regular expressions with Humanitarian Event details',
#     'Updated data search regular expressions with Humanitarian Event details',
#     'Failed to Update data search regular expressions with Humanitarian Event details'
# ),

# #  - search for data
# #  - check schema of data
# #  - check for multiple shapefile matches
# #  - calucate data-source credit text
# # Get Template
# # Get Oritenation
# # 'cook()'
# # - set zoom
# # alignLegend
# # Export
# #  - Create export dir
# #  - do_export jpeg, pdf, thumbnail
# #  - export_atlas (if required)
# #  - create zip file
# # Upload to MDR
