h1. What is the problem?

Could not locate gis data to add to this map layer: *<%layer.name%>* 

The expected data files name should match this pattern (as a 'Regular Expression'):

{{<%layer.reg_exp%>}}

<%#search_info.is_parsable%>
In more human readable terms, this means that these bits of the filename must match exactly these values:

<%#search_info.valid_clause_list%>
* {{<%&valid_clause%>}}
<%/search_info.valid_clause_list%>

Whilst _in general_ any valid value is acceptable for these part of the filename:

<%#search_info.invalid_clause_list %>
* {{<%&invalid_clause%>}}
<%/search_info.invalid_clause_list %>
<%/search_info.is_parsable%>


h1. What is the likely consequence of this problem? 

Any automated maps will be produced with this layer missing. The usefulness of these maps may be greatly reduced.

h1. How can this problem be resolved?

The relevant data must be located and correctly named.

* Check the Data Scramble coordination notes. Check whether this data has been obtained within the data scramble.
  If not, follow the instructions for downloading and preparing this data.
* Check any other JIRA tasks on this board. It may be that this data exists but it is incorrectly named. For
  information related this specific layer see this page:
  [https://wiki.mapaction.org/x/QIDW|https://wiki.mapaction.org/x/QIDW]
  
h1. Other information
 
|layer name|{{<%layer.name%>}}|
|data search regex|{{<%layer.reg_exp%>}}|
|Full path to layer file|{{<%layer.layer_file_path%>}}|